package org.jtheque.memory;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdesktop.swingx.event.WeakEventListenerList;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.module.annotations.ModuleDefinition;
import org.jtheque.core.managers.module.annotations.Module;
import org.jtheque.core.managers.module.annotations.Plug;
import org.jtheque.core.managers.module.annotations.UnPlug;
import org.jtheque.core.managers.view.able.components.StateBarComponent;
import org.jtheque.core.managers.view.able.components.StateBarComponent.Position;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.utils.bean.Version;

import java.lang.management.ManagementFactory;
import java.lang.management.MemoryUsage;
import java.util.Timer;
import java.util.TimerTask;

/**
 * A module to display the memory used/committed in the state bar.
 *
 * @author Baptiste Wicht
 */
@Module(name = "JTheque Memory Module", author = "Baptiste Wicht", description = "Affiche la mmoire utilise dans la barre d'tat. ", 
    version = "1.4", coreVersion = "2.0.1", jarFile = "jtheque-memory-module-1.4.jar")
@ModuleDefinition(updateURL = "http://jtheque.developpez.com/public/versions/MemoryModule.versions")
public final class MemoryModule {
    /**
     * The timer to manager the memory task.
     */
    private static final Timer memoryManagementTimer = new Timer();

    /**
     * The list to manage the listeners.
     */
    private static final WeakEventListenerList listenerList = new WeakEventListenerList();

    /**
     * The statebar component who display the memory.
     */
    private StateBarComponent component;

    /**
     * The basename for the ressources.
     */
    public static final String BASENAME = "org/jtheque/memory/resources";

    @Plug
    public void plug() {
        memoryManagementTimer.scheduleAtFixedRate(new MemoryTask(), 1000 * 15, 1000 * 60);

        component = new StateBarComponent(new MemoryPanel(), Position.RIGHT);

        Managers.getManager(IViewManager.class).addStateBarComponent(component);
    }

    @UnPlug
    public void unplug() {
        Managers.getManager(IViewManager.class).removeStateBarComponent(component);

        if (memoryManagementTimer != null) {
            memoryManagementTimer.cancel();
        }

        for(MemoryListener l : listenerList.getListeners(MemoryListener.class)){
            removeMemoryListener(l);
        }
    }

    /**
     * Add a memory listener.
     *
     * @param listener The listener to add.
     */
    public static void addMemoryListener(MemoryListener listener) {
        listenerList.add(MemoryListener.class, listener);
    }

    /**
     * Remove a memory listener.
     *
     * @param listener The listener to remove
     */
    public static void removeMemoryListener(MemoryListener listener) {
        listenerList.remove(MemoryListener.class, listener);
    }

    /**
     * Fire a memory event when the function has been updated.
     */
    private static void fireMemoryChanged() {
        MemoryListener[] listeners = listenerList.getListeners(MemoryListener.class);

        for (MemoryListener listener : listeners) {
            listener.memoryUsageChanged();
        }
    }

    /**
     * Return the memory usage of the application.
     *
     * @return The current memory usage.
     */
    private static MemoryUsage getMemoryUsage() {
        return ManagementFactory.getMemoryMXBean().getNonHeapMemoryUsage();
    }

    /**
     * Return the memory used.
     *
     * @return The amount of memory used by the application.
     */
    public static long getMemoryUsed() {
        return getMemoryUsage().getUsed();
    }

    /**
     * Return the memory committed.
     *
     * @return The amount of memory committed by the application.
     */
    public static long getMemoryCommitted() {
        return getMemoryUsage().getCommitted();
    }

    /**
     * Clean the memory, force run the gc.
     */
    public static void cleanMemory() {
        for (int i = 0; i < 5; i++) {
            System.gc();
        }

        memoryManagementTimer.schedule(new MemoryTask(), 5);
    }

    /**
     * A timer task to calculate the memory usage and avert the listeners of the change.
     *
     * @author Baptiste Wicht
     */
    private static class MemoryTask extends TimerTask{
		@Override
		public void run() {
			fireMemoryChanged();
		}
	}
}